#include "../route.h"
#include <string>
#include "AtomicAction.h"
#include "Action.h"
#include <assert.h>

CAction::CAction( bool _permit )
	:permit( _permit ) {
}

CAction::CAction( bool _permit, const std::vector<CAtomicAction> & _atoms )
	:permit( _permit )
{
	assert( _permit || (!_permit && _atoms.empty()));
	atoms = _atoms;
}

CAction::~CAction(void)
{
}

void CAction::addAtom( const CAtomicAction & new_atom )
{
	atoms.push_back( new_atom );
}

bool CAction::applyTo( Route & route )
{
	if ( permit ) {
		// perform attribute manipulation, if there are any atomic actions
		if ( !atoms.empty() ) {
			for ( size_t i=0; i < atoms.size(); i++ ) {
				atoms[i].applyTo( route );
			}
		}
		return true;
	} else {
		return false;
	}
}

std::string CAction::toString( const std::string & input )
{
	std::string str = input + "action:\n";
	if ( permit ) {
		str += input + "  permit";
	} else {
		str += input + "  deny\n";
		return str;
	}
	if ( !atoms.empty() ) {
		for ( size_t i=0; i < atoms.size(); i++ ) {
			str += " AND\n"+ atoms[i].toString(input+"  ") +"\n";
		}
	}
	return str;
}
