#include "../route.h"
#include <string>
#include "AtomicAction.h"
#include <assert.h>

using namespace std;

const string CAtomicAction::ACTION_NAMES [] = { "set", "prepend", "append" };


CAtomicAction::CAtomicAction( int attrib_val, bool using_nhi, int action_val, std::string val )
	:use_NHI( using_nhi ),
	attribute_type( attrib_val ),
	action_type( action_val ),
	value( val ) {
}


CAtomicAction::~CAtomicAction(void)
{
}


void 
CAtomicAction::applyTo( Route & route ) {
	switch( attribute_type ) {
	case Origin::TYPECODE:
		switch( action_type ) {
			default:
				assert(0);//undefined action for Origin attribute
		}
		break;
	case ASpath::TYPECODE:
		switch( action_type ) {
		case PREPEND:
			{
				// 			if (Global.linked_aspaths) {
// 				Debug.gerr("cannot modify AS path when " +
// 					"linked_aspaths option is in use");
// 				}
				if ( value.empty() ) {
					assert(0); // no entry;
				}
				route.prepend_as( value.c_str() );
				break;
			}
		default:
			assert(0); //("undefined action for ASpath attribute: " +
		}
		break;
	case NextHop::TYPECODE:
		switch( action_type ) {
		default:
			assert(0); //("undefined action for NextHop attribute: " 
		}
		break;
	case MED::TYPECODE:
		switch( action_type ) {
		case SET:
			{
				int med_value = atoi ( value.c_str() );
				route.set_med( med_value );
				break;
			}
		default:
			assert(0); //"undefined action for MED attribute: "
		}
		break;
	case LocalPref::TYPECODE:
		switch (action_type) {
		case SET:
			{
				int pref_value = atoi( value.c_str() );
				route.set_localpref( pref_value );
				break;
			}
		default:
			assert(0); //"undefined action for LocalPref attribute: " 
		}
		break;
	case AtomicAggregate::TYPECODE:
		switch (action_type) {
		default:
			assert(0); //"undefined action for AtomicAggregate attribute: 
		}
		break;
	case Aggregator::TYPECODE:
		switch (action_type) {
		default:
			assert(0); //"undefined action for Aggregator attribute: " 
		}
		break;
// 	case Communities.TYPECODE:
// 		switch (action_type) {
// 	default:
// 		Debug.gexcept("undefined action for Communities attribute: " +
// 			actionnames[action_type]);
// 		}
// 		break;
	case OriginatorID::TYPECODE:
		switch (action_type) {
		default:
			assert(0); //"undefined action for OriginatorID attribute: " 
		}
		break;
	case ClusterList::TYPECODE:
		switch (action_type) {
		default:
			assert(0); //"undefined action for ClusterList attribute: " 
		}
		break;
	default:
		assert(0); //"unrecognized path attribute type: " + attrib_type);
	}
}


std::string 
CAtomicAction::toString( const std::string & input ) {
	string str = input + "(";

	str += ACTION_NAMES[action_type];
	str += value;

	str += ")";
	return str;
}

