#include "../route.h"
#include "AtomicPredicate.h"
#include "../Path/nexthop.h"
#include <assert.h>
#include "Utils.h"


CAtomicPredicate::CAtomicPredicate( int attribute_val, 
								   const std::string & matcher )
   :use_NHI( false ),
   attribute_type( attribute_val ) {
	   
	   //if ( attribute_val == ASpath::TYPECODE || attribute_val == NextHop::TYPECODE) {
		   tre_regcomp( &regular_expr_proc, matcher.c_str(), REG_EXTENDED );		   
	   //}
	   match_string = matcher;
}


CAtomicPredicate::~CAtomicPredicate(void)
{
}


bool CAtomicPredicate::applyTo( Route & route )
{
	switch ( attribute_type ) {
	case Route::NLRI:
		{
			std::string nlri_string = route.nlri->toString();
			if ( tre_regexec( &regular_expr_proc, nlri_string.c_str(), 
					0, 0, REG_NOSUB ) == REG_OK ) {
					return true;
			} else
				return false;
			//return ( match_string.find( nlri_string ) != std::string::npos );
		}
	case Origin::TYPECODE:
		{
			int origin = route.origin();
			//illegal 'matcher' value for 'origin' (must be egp, igp, or inc)
			assert ( match_string.compare("igp") != 0 &&
				match_string.compare("egp") != 0 &&
				match_string.compare("inc") != 0 );

			if ( origin == Origin::IGP && (match_string.compare("igp") == 0) ||
				origin == Origin::EGP && (match_string.compare("egp") == 0) ||
				origin == Origin::INC && (match_string.compare("inc") ==0) ) {
					return true;
			} else {
				return false;
			}
		}
	case ASpath::TYPECODE:
		{
			// We pad the AS path string so that it begins and ends with a space.
			// This greatly simplifies the regular expressions that need to be
			// specified for most predicates.
			char * aspath_min_string = new char [ route.pas_bytecount() + 1 ];
			route.pasToStream( aspath_min_string );	
			if ( tre_regexec( &regular_expr_proc, aspath_min_string, 0, 0, REG_NOSUB ) == REG_OK ) {
					return true;
			} else {
				return false;
			}
			//return ( match_string.find( aspath_min_string ) != std::string::npos );
		}
	case NextHop::TYPECODE:
		{
			string nexthopstr = route.nexthop()->toString();
			if ( tre_regexec( &regular_expr_proc, nexthopstr.c_str(), 
				0, 0, REG_NOSUB ) == REG_OK ) {
					return true;
			} else {
				return false;
			}
			//return ( ma.tch_string.find( nexthopstr ) != string::npos );
		}
	case MED::TYPECODE:
		//("the MED path attribute is not currently implemented");
		return false;
	case LocalPref::TYPECODE:
		{
			bool has_localpref = route.has_localpref();
			if ( !has_localpref ) { // no local pref
				if ( match_string.empty() ) {
					return true;
				} else {
					return false;
				}
			} else { // has local pref
				std::string local_pref_str = Utils::converToString( route.localpref() );
				return ( match_string.find( local_pref_str ) != string::npos );
			}
		}
	case AtomicAggregate::TYPECODE:
		//("matching based on the Atomic Aggregate path attribute " + "is not currently implemented");
		return false;
	case Aggregator::TYPECODE:
		//Debug.gwarn("matching based on the Aggregator path attribute " + "is not currently implemented");
		return false;
	/*case Communities.TYPECODE:
		Debug.gwarn("the Communities path attribute is not " +
			"currently implemented");
		return false;*/
	case OriginatorID::TYPECODE:
		{
			if ( route.has_orig_id() ) {
				string originatorstr = route.orig_id()->toString();
				if ( tre_regexec( &regular_expr_proc, originatorstr.c_str(), 
					0, 0, REG_NOSUB ) == REG_OK ) {
						return true;
				} else {
					return false;
				}
				//return ( match_string.find( originatorstr ) != string::npos );
			} else { // no originator attribute
				return false;
			}
		}
	case ClusterList::TYPECODE:
		{
			if ( !route.has_cluster_list() ) { // route has no cluster list path attribute
				return false;
			}
			// We pad the cluster list string so that it begins and ends with a
			// space.  This greatly simplifies the regular expressions that need to
			// be specified for most predicates.
			string clusterlist_str = route.cluster_list()->toString();
			if ( tre_regexec( &regular_expr_proc, clusterlist_str.c_str(), 
				0, 0, REG_NOSUB ) == REG_OK ) {
					return true;
			} else {
				return false;
			}
			return ( match_string.find( clusterlist_str ) != string::npos );
		}
	default:
		//("unrecognized path attribute type: " + attrib_type);
		return false;
	}
}


std::string 
CAtomicPredicate::toString( const std::string & input )
{
	std::string attribute_str = Utils::converToString( attribute_type );
	return input + attribute_str + " has matcher \"" +
		match_string + "\"";
}
