#include "../route.h"
#include "AtomicAction.h"
#include "Action.h"
#include "AtomicPredicate.h"
#include "Predicate.h"
#include "Clause.h"
#include "Rule.h"
#include "CommunityList.h"
#include "Utils.h"

CCommunityList::CCommunityList( const std::string & _list_name, 
							   bool _permit, 
							   int communitynumber /*= -1*/, 
							   const std::string & reg_expr /*= "" */ )
	:list_name( _list_name ),
	list_number( -1 )
{
	createList( _permit, communitynumber, reg_expr );
}

CCommunityList::CCommunityList( int listnumber, 
							   bool _permit, 
							   int communitynumber /*= -1*/, 
							   const std::string & reg_expr /*= "" */ ) 
	:list_number( listnumber ),
	list_name( "" )
{
	createList( _permit, communitynumber, reg_expr );
}


void 
CCommunityList::createList( bool _permit, 
						   int communitynumber /*= -1*/, 
						   const std::string & reg_expr /*= "" */ ) {
	   rule = CRule( );
	   std::string predicate_string = "";
	   if ( communitynumber != -1 ) {
		   predicate_string = Utils::converToString( communitynumber );
	   } else {
		   predicate_string.append( reg_expr );
	   }
	   CAtomicPredicate atomic_predicate( Community::TYPECODE, predicate_string );
	   CPredicate predicate;
	   predicate.addAtom( atomic_predicate );
	   rule.addClause( CClause( predicate, CAction( _permit ) ) );
}


CCommunityList::~CCommunityList(void) {
}


bool CCommunityList::applyTo( Route & route ) {
	return rule.applyTo( route );
}


std::string CCommunityList::toString( const std::string & input )
{
	std::string str = input + "community list:\n";
	if ( list_number != -1 ) {
		std::string list_number_str = Utils::converToString( list_number );
		str += list_number_str;
	} else {
		if ( !list_name.empty() )
			str += list_name;
	}
	str += rule.toString();
	return str;
}


