#include <string>
#include "AtomicAction.h"
#include "Action.h"
#include "AtomicPredicate.h"
#include "Predicate.h"
#include "Clause.h"
#include "Rule.h"

CRule::CRule ( bool permit ) {
	if ( permit ) {
		addClause( CClause(CPredicate(std::vector<CAtomicPredicate>()), CAction(true, std::vector<CAtomicAction>())));
	}
}


CRule::CRule( const std::vector<CClause> & clause_list )
: clauses( clause_list ) {
}


CRule::~CRule(void) {
}


bool 
CRule::applyTo( Route & route ) {
	if ( !clauses.empty() ) {
		for ( size_t i = 0; i < clauses.size(); i++ ) {
			bool * results  = 0;
			results = clauses[i].applyTo( route );
			if ( results && results[0] ) { // whether or not the clause's predicate matched
				return results[1]; // whether or not the route was permitted
			}
			delete []results;
		}
	}
	return false; // no clause's predicate matched, so deny
}

std::string 
CRule::toString() {
	std::string str = "policy rule:\n";
	for ( size_t i = 0; i < clauses.size(); i++ ) {
		str += clauses[i].toString("  ");
	}
	return str;
}
